@@ -6,10 +6,12 @@
 // spell-checker:ignore (chrono) Datelike Timelike ; (format) DATEFILE MMDDhhmm ; (vars) datetime datetimes
 
 use chrono::format::{Item, StrftimeItems};
-use chrono::{DateTime, FixedOffset, Local, Offset, TimeDelta, Utc};
+use chrono::{DateTime, FixedOffset, Local, Offset, TimeDelta, TimeZone, Utc};
 #[cfg(windows)]
 use chrono::{Datelike, Timelike};
+use chrono_tz::{OffsetName, Tz};
 use clap::{crate_version, Arg, ArgAction, Command};
+use iana_time_zone::get_timezone;
 #[cfg(all(unix, not(target_os = "macos"), not(target_os = "redox")))]
 use libc::{clock_settime, timespec, CLOCK_REALTIME};
 use std::fs::File;
@@ -91,6 +93,7 @@ enum DateSource {
     Now,
     Custom(String),
     File(PathBuf),
+    Stdin,
     Human(TimeDelta),
 }
 
@@ -102,7 +105,7 @@ enum Iso8601Format {
     Ns,
 }
 
-impl<'a> From<&'a str> for Iso8601Format {
+impl From<&str> for Iso8601Format {
     fn from(s: &str) -> Self {
         match s {
             HOURS => Self::Hours,
@@ -122,7 +125,7 @@ enum Rfc3339Format {
     Ns,
 }
 
-impl<'a> From<&'a str> for Rfc3339Format {
+impl From<&str> for Rfc3339Format {
     fn from(s: &str) -> Self {
         match s {
             DATE => Self::Date,
@@ -173,7 +176,10 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
             DateSource::Custom(date.into())
         }
     } else if let Some(file) = matches.get_one::<String>(OPT_FILE) {
-        DateSource::File(file.into())
+        match file.as_ref() {
+            "-" => DateSource::Stdin,
+            _ => DateSource::File(file.into()),
+        }
     } else {
         DateSource::Now
     };
@@ -223,11 +229,9 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
                 Box::new(iter)
             }
             DateSource::Human(relative_time) => {
-                // Get the current DateTime<FixedOffset> for things like "1 year ago"
-                let current_time = DateTime::<FixedOffset>::from(Local::now());
-                // double check the result is overflow or not of the current_time + relative_time
+                // Double check the result is overflow or not of the current_time + relative_time
                 // it may cause a panic of chrono::datetime::DateTime add
-                match current_time.checked_add_signed(relative_time) {
+                match now.checked_add_signed(relative_time) {
                     Some(date) => {
                         let iter = std::iter::once(Ok(date));
                         Box::new(iter)
@@ -235,11 +239,16 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
                     None => {
                         return Err(USimpleError::new(
                             1,
-                            format!("invalid date {}", relative_time),
+                            format!("invalid date {relative_time}"),
                         ));
                     }
                 }
             }
+            DateSource::Stdin => {
+                let lines = BufReader::new(std::io::stdin()).lines();
+                let iter = lines.map_while(Result::ok).map(parse_date);
+                Box::new(iter)
+            }
             DateSource::File(ref path) => {
                 if path.is_dir() {
                     return Err(USimpleError::new(
@@ -265,8 +274,21 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
         for date in dates {
             match date {
                 Ok(date) => {
+                    // TODO - Revisit when chrono 0.5 is released. https://github.com/chronotope/chrono/issues/970
+                    let tz = match std::env::var("TZ") {
+                        // TODO Support other time zones...
+                        Ok(s) if s == "UTC0" || s.is_empty() => Tz::Etc__UTC,
+                        _ => match get_timezone() {
+                            Ok(tz_str) => tz_str.parse().unwrap(),
+                            Err(_) => Tz::Etc__UTC,
+                        },
+                    };
+                    let offset = tz.offset_from_utc_date(&Utc::now().date_naive());
+                    let tz_abbreviation = offset.abbreviation();
                     // GNU `date` uses `%N` for nano seconds, however crate::chrono uses `%f`
-                    let format_string = &format_string.replace("%N", "%f");
+                    let format_string = &format_string
+                        .replace("%N", "%f")
+                        .replace("%Z", tz_abbreviation.unwrap_or("UTC"));
                     // Refuse to pass this string to chrono as it is crashing in this crate
                     if format_string.contains("%#z") {
                         return Err(USimpleError::new(
@@ -396,7 +418,7 @@ fn make_format_string(settings: &Settings) -> &str {
             Rfc3339Format::Ns => "%F %T.%f%:z",
         },
         Format::Custom(ref fmt) => fmt,
-        Format::Default => "%c",
+        Format::Default => "%a %b %e %X %Z %Y",
     }
 }
 
@@ -404,9 +426,8 @@ fn make_format_string(settings: &Settings) -> &str {
 /// If it fails, return a tuple of the `String` along with its `ParseError`.
 fn parse_date<S: AsRef<str> + Clone>(
     s: S,
-) -> Result<DateTime<FixedOffset>, (String, chrono::format::ParseError)> {
-    // TODO: The GNU date command can parse a wide variety of inputs.
-    s.as_ref().parse().map_err(|e| (s.as_ref().into(), e))
+) -> Result<DateTime<FixedOffset>, (String, parse_datetime::ParseDateTimeError)> {
+    parse_datetime::parse_datetime(s.as_ref()).map_err(|e| (s.as_ref().into(), e))
 }
 
 #[cfg(not(any(unix, windows)))]
