@@ -2,6 +2,9 @@
 //
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
+
+// spell-checker:ignore defg
+
 use crate::common::util::TestScenario;
 
 static INPUT: &str = "lists.txt";
@@ -43,6 +46,13 @@ static COMPLEX_SEQUENCE: &TestedSequence = &TestedSequence {
     sequence: "9-,6-7,-2,4",
 };
 
+#[test]
+fn test_no_args() {
+    new_ucmd!().fails().stderr_is(
+        "cut: invalid usage: expects one of --fields (-f), --chars (-c) or --bytes (-b)\n",
+    );
+}
+
 #[test]
 fn test_invalid_arg() {
     new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
@@ -118,7 +128,26 @@ fn test_whitespace_with_char() {
 }
 
 #[test]
-fn test_specify_delimiter() {
+fn test_delimiter_with_byte_and_char() {
+    for conflicting_arg in ["-c", "-b"] {
+        new_ucmd!()
+            .args(&[conflicting_arg, COMPLEX_SEQUENCE.sequence, "-d="])
+            .fails()
+            .stderr_is("cut: invalid input: The '--delimiter' ('-d') option only usable if printing a sequence of fields\n")
+            .code_is(1);
+    }
+}
+
+#[test]
+fn test_too_large() {
+    new_ucmd!()
+        .args(&["-b1-18446744073709551615", "/dev/null"])
+        .fails()
+        .code_is(1);
+}
+
+#[test]
+fn test_delimiter() {
     for param in ["-d", "--delimiter", "--del"] {
         new_ucmd!()
             .args(&[param, ":", "-f", COMPLEX_SEQUENCE.sequence, INPUT])
@@ -127,6 +156,15 @@ fn test_specify_delimiter() {
     }
 }
 
+#[test]
+fn test_delimiter_with_more_than_one_char() {
+    new_ucmd!()
+        .args(&["-d", "ab", "-f1"])
+        .fails()
+        .stderr_contains("cut: the delimiter must be a single character")
+        .no_stdout();
+}
+
 #[test]
 fn test_output_delimiter() {
     // we use -d here to ensure output delimiter
@@ -195,46 +233,148 @@ fn test_zero_terminated_only_delimited() {
 }
 
 #[test]
-fn test_directory_and_no_such_file() {
+fn test_is_a_directory() {
     let (at, mut ucmd) = at_and_ucmd!();
 
     at.mkdir("some");
 
     ucmd.arg("-b1")
         .arg("some")
-        .run()
+        .fails()
+        .code_is(1)
         .stderr_is("cut: some: Is a directory\n");
+}
 
+#[test]
+fn test_no_such_file() {
     new_ucmd!()
         .arg("-b1")
         .arg("some")
-        .run()
+        .fails()
+        .code_is(1)
         .stderr_is("cut: some: No such file or directory\n");
 }
 
 #[test]
-fn test_equal_as_delimiter1() {
-    new_ucmd!()
-        .args(&["-f", "2", "-d="])
-        .pipe_in("--dir=./out/lib")
-        .succeeds()
-        .stdout_only("./out/lib\n");
+fn test_equal_as_delimiter() {
+    for arg in ["-d=", "--delimiter=="] {
+        new_ucmd!()
+            .args(&["-f2", arg])
+            .pipe_in("--dir=./out/lib")
+            .succeeds()
+            .stdout_only("./out/lib\n");
+    }
 }
 
 #[test]
-fn test_equal_as_delimiter2() {
-    new_ucmd!()
-        .args(&["-f2", "--delimiter="])
-        .pipe_in("a=b\n")
-        .succeeds()
-        .stdout_only("a=b\n");
+fn test_empty_string_as_delimiter() {
+    for arg in ["-d''", "--delimiter=", "--delimiter=''"] {
+        new_ucmd!()
+            .args(&["-f2", arg])
+            .pipe_in("a\0b\n")
+            .succeeds()
+            .stdout_only("b\n");
+    }
 }
 
 #[test]
-fn test_equal_as_delimiter3() {
+fn test_empty_string_as_delimiter_with_output_delimiter() {
     new_ucmd!()
         .args(&["-f", "1,2", "-d", "''", "--output-delimiter=Z"])
         .pipe_in("ab\0cd\n")
         .succeeds()
         .stdout_only_bytes("abZcd\n");
 }
+
+#[test]
+fn test_newline_as_delimiter() {
+    for (field, expected_output) in [("1", "a:1\n"), ("2", "b:\n")] {
+        new_ucmd!()
+            .args(&["-f", field, "-d", "\n"])
+            .pipe_in("a:1\nb:")
+            .succeeds()
+            .stdout_only_bytes(expected_output);
+    }
+}
+
+#[test]
+fn test_newline_as_delimiter_with_output_delimiter() {
+    new_ucmd!()
+        .args(&["-f1-", "-d", "\n", "--output-delimiter=:"])
+        .pipe_in("a\nb\n")
+        .succeeds()
+        .stdout_only_bytes("a:b\n");
+}
+
+#[test]
+fn test_multiple_delimiters() {
+    new_ucmd!()
+        .args(&["-f2", "-d:", "-d="])
+        .pipe_in("a:=b\n")
+        .succeeds()
+        .stdout_only("b\n");
+
+    new_ucmd!()
+        .args(&["-f2", "-d=", "-d:"])
+        .pipe_in("a:=b\n")
+        .succeeds()
+        .stdout_only("=b\n");
+}
+
+#[test]
+fn test_multiple_mode_args() {
+    for args in [
+        vec!["-b1", "-b2"],
+        vec!["-c1", "-c2"],
+        vec!["-f1", "-f2"],
+        vec!["-b1", "-c2"],
+        vec!["-b1", "-f2"],
+        vec!["-c1", "-f2"],
+        vec!["-b1", "-c2", "-f3"],
+    ] {
+        new_ucmd!()
+        .args(&args)
+        .fails()
+        .stderr_is("cut: invalid usage: expects no more than one of --fields (-f), --chars (-c) or --bytes (-b)\n");
+    }
+}
+
+#[test]
+#[cfg(unix)]
+fn test_8bit_non_utf8_delimiter() {
+    use std::ffi::OsStr;
+    use std::os::unix::ffi::OsStrExt;
+    let delim = OsStr::from_bytes(b"\xAD".as_slice());
+    new_ucmd!()
+        .arg("-d")
+        .arg(delim)
+        .args(&["--out=_", "-f2,3", "8bit-delim.txt"])
+        .succeeds()
+        .stdout_check(|out| out == "b_c\n".as_bytes());
+}
+
+#[test]
+fn test_newline_preservation_with_f1_option() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    at.write("1", "a\nb");
+    let expected = "a\nb\n";
+    ucmd.args(&["-f1-", "1"]).succeeds().stdout_is(expected);
+}
+
+#[test]
+fn test_output_delimiter_with_character_ranges() {
+    new_ucmd!()
+        .args(&["-c2-3,4-", "--output-delim=:"])
+        .pipe_in("abcdefg\n")
+        .succeeds()
+        .stdout_only("bc:defg\n");
+}
+
+#[test]
+fn test_output_delimiter_with_adjacent_ranges() {
+    new_ucmd!()
+        .args(&["-b1-2,3-4", "--output-d=:"])
+        .pipe_in("abcd\n")
+        .succeeds()
+        .stdout_only("ab:cd\n");
+}
