@@ -396,7 +396,7 @@ fn test_null_stats() {
         .arg("if=null.txt")
         .run()
         .stderr_contains("0+0 records in\n0+0 records out\n0 bytes copied, ")
-        .stderr_matches(&Regex::new(r"\d\.\d+(e-\d\d)? s, ").unwrap())
+        .stderr_matches(&Regex::new(r"\d(\.\d+)?(e-\d\d)? s, ").unwrap())
         .stderr_contains("0.0 B/s")
         .success();
 }
@@ -1649,7 +1649,7 @@ fn test_reading_partial_blocks_from_fifo() {
     // until the writer process starts).
     let mut reader_command = Command::new(TESTS_BINARY);
     let child = reader_command
-        .args(["dd", "ibs=3", "obs=3", &format!("if={}", fifoname)])
+        .args(["dd", "ibs=3", "obs=3", &format!("if={fifoname}")])
         .stdout(Stdio::piped())
         .stderr(Stdio::piped())
         .spawn()
@@ -1658,13 +1658,14 @@ fn test_reading_partial_blocks_from_fifo() {
     // Start different processes to write to the FIFO, with a small
     // pause in between.
     let mut writer_command = Command::new("sh");
-    writer_command
+    let _ = writer_command
         .args([
             "-c",
-            &format!("(printf \"ab\"; sleep 0.1; printf \"cd\") > {}", fifoname),
+            &format!("(printf \"ab\"; sleep 0.1; printf \"cd\") > {fifoname}"),
         ])
         .spawn()
-        .unwrap();
+        .unwrap()
+        .wait();
 
     let output = child.wait_with_output().unwrap();
     assert_eq!(output.stdout, b"abcd");
@@ -1692,7 +1693,7 @@ fn test_reading_partial_blocks_from_fifo_unbuffered() {
     // `bs=N` takes precedence over `ibs=N` and `obs=N`.
     let mut reader_command = Command::new(TESTS_BINARY);
     let child = reader_command
-        .args(["dd", "bs=3", "ibs=1", "obs=1", &format!("if={}", fifoname)])
+        .args(["dd", "bs=3", "ibs=1", "obs=1", &format!("if={fifoname}")])
         .stdout(Stdio::piped())
         .stderr(Stdio::piped())
         .spawn()
@@ -1701,13 +1702,14 @@ fn test_reading_partial_blocks_from_fifo_unbuffered() {
     // Start different processes to write to the FIFO, with a small
     // pause in between.
     let mut writer_command = Command::new("sh");
-    writer_command
+    let _ = writer_command
         .args([
             "-c",
-            &format!("(printf \"ab\"; sleep 0.1; printf \"cd\") > {}", fifoname),
+            &format!("(printf \"ab\"; sleep 0.1; printf \"cd\") > {fifoname}"),
         ])
         .spawn()
-        .unwrap();
+        .unwrap()
+        .wait();
 
     let output = child.wait_with_output().unwrap();
     assert_eq!(output.stdout, b"abcd");
@@ -1726,7 +1728,26 @@ fn test_iflag_directory_fails_when_file_is_passed_via_std_in() {
         .args(&["iflag=directory", "count=0"])
         .set_stdin(std::process::Stdio::from(File::open(filename).unwrap()))
         .fails()
-        .stderr_contains("standard input: not a directory");
+        .stderr_only("dd: setting flags for 'standard input': Not a directory\n");
+}
+
+#[test]
+#[cfg(any(target_os = "linux", target_os = "android"))]
+fn test_iflag_directory_passes_when_dir_is_redirected() {
+    new_ucmd!()
+        .args(&["iflag=directory", "count=0"])
+        .set_stdin(std::process::Stdio::from(File::open(".").unwrap()))
+        .succeeds();
+}
+
+#[test]
+#[cfg(any(target_os = "linux", target_os = "android"))]
+fn test_iflag_directory_fails_when_file_is_piped_via_std_in() {
+    new_ucmd!()
+        .arg("iflag=directory")
+        .pipe_in("")
+        .fails()
+        .stderr_only("dd: setting flags for 'standard input': Not a directory\n");
 }
 
 #[test]
@@ -1769,9 +1790,22 @@ fn test_stdin_stdout_not_rewound_even_when_connected_to_seekable_file() {
         .succeeds();
 
     let err_file_content = std::fs::read_to_string(at.plus_as_string("err")).unwrap();
-    println!("stderr:\n{}", err_file_content);
+    println!("stderr:\n{err_file_content}");
 
     let out_file_content = std::fs::read_to_string(at.plus_as_string("out")).unwrap();
-    println!("stdout:\n{}", out_file_content);
+    println!("stdout:\n{out_file_content}");
     assert_eq!(out_file_content, "bde");
 }
+
+#[test]
+fn test_wrong_number_err_msg() {
+    new_ucmd!()
+        .args(&["count=kBb"])
+        .fails()
+        .stderr_contains("dd: invalid number: ‘kBb’\n");
+
+    new_ucmd!()
+        .args(&["count=1kBb555"])
+        .fails()
+        .stderr_contains("dd: invalid number: ‘1kBb555’\n");
+}
