@@ -1,3 +1,7 @@
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 use crate::common::util::TestScenario;
 
 const DIR: &str = "dir";
@@ -25,6 +29,14 @@ fn test_invalid_arg() {
     new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
 }
 
+#[test]
+fn test_no_arg() {
+    new_ucmd!()
+        .fails()
+        .code_is(1)
+        .stderr_contains("error: the following required arguments were not provided:");
+}
+
 #[test]
 fn test_rmdir_empty_directory_no_parents() {
     let (at, mut ucmd) = at_and_ucmd!();
