@@ -48,12 +48,12 @@ fn test_hex_rejects_sign_after_identifier() {
         .args(&["-0x-123ABC"])
         .fails()
         .no_stdout()
-        .stderr_contains("unexpected argument '-0' found");
+        .usage_error("invalid floating point argument: '-0x-123ABC'");
     new_ucmd!()
         .args(&["-0x+123ABC"])
         .fails()
         .no_stdout()
-        .stderr_contains("unexpected argument '-0' found");
+        .usage_error("invalid floating point argument: '-0x+123ABC'");
 }
 
 #[test]
@@ -303,18 +303,15 @@ fn test_preserve_negative_zero_start() {
     new_ucmd!()
         .args(&["-0", "1"])
         .succeeds()
-        .stdout_is("-0\n1\n")
-        .no_stderr();
+        .stdout_only("-0\n1\n");
     new_ucmd!()
         .args(&["-0", "1", "2"])
         .succeeds()
-        .stdout_is("-0\n1\n2\n")
-        .no_stderr();
+        .stdout_only("-0\n1\n2\n");
     new_ucmd!()
         .args(&["-0", "1", "2.0"])
         .succeeds()
-        .stdout_is("-0\n1\n2\n")
-        .no_stderr();
+        .stdout_only("-0\n1\n2\n");
 }
 
 #[test]
@@ -322,8 +319,7 @@ fn test_drop_negative_zero_end() {
     new_ucmd!()
         .args(&["1", "-1", "-0"])
         .succeeds()
-        .stdout_is("1\n0\n")
-        .no_stderr();
+        .stdout_only("1\n0\n");
 }
 
 #[test]
@@ -331,8 +327,11 @@ fn test_width_scientific_notation() {
     new_ucmd!()
         .args(&["-w", "999", "1e3"])
         .succeeds()
-        .stdout_is("0999\n1000\n")
-        .no_stderr();
+        .stdout_only("0999\n1000\n");
+    new_ucmd!()
+        .args(&["-w", "999", "1E3"])
+        .succeeds()
+        .stdout_only("0999\n1000\n");
 }
 
 #[test]
@@ -340,18 +339,15 @@ fn test_width_negative_zero() {
     new_ucmd!()
         .args(&["-w", "-0", "1"])
         .succeeds()
-        .stdout_is("-0\n01\n")
-        .no_stderr();
+        .stdout_only("-0\n01\n");
     new_ucmd!()
         .args(&["-w", "-0", "1", "2"])
         .succeeds()
-        .stdout_is("-0\n01\n02\n")
-        .no_stderr();
+        .stdout_only("-0\n01\n02\n");
     new_ucmd!()
         .args(&["-w", "-0", "1", "2.0"])
         .succeeds()
-        .stdout_is("-0\n01\n02\n")
-        .no_stderr();
+        .stdout_only("-0\n01\n02\n");
 }
 
 #[test]
@@ -359,33 +355,27 @@ fn test_width_negative_zero_decimal_notation() {
     new_ucmd!()
         .args(&["-w", "-0.0", "1"])
         .succeeds()
-        .stdout_is("-0.0\n01.0\n")
-        .no_stderr();
+        .stdout_only("-0.0\n01.0\n");
     new_ucmd!()
         .args(&["-w", "-0.0", "1.0"])
         .succeeds()
-        .stdout_is("-0.0\n01.0\n")
-        .no_stderr();
+        .stdout_only("-0.0\n01.0\n");
     new_ucmd!()
         .args(&["-w", "-0.0", "1", "2"])
         .succeeds()
-        .stdout_is("-0.0\n01.0\n02.0\n")
-        .no_stderr();
+        .stdout_only("-0.0\n01.0\n02.0\n");
     new_ucmd!()
         .args(&["-w", "-0.0", "1", "2.0"])
         .succeeds()
-        .stdout_is("-0.0\n01.0\n02.0\n")
-        .no_stderr();
+        .stdout_only("-0.0\n01.0\n02.0\n");
     new_ucmd!()
         .args(&["-w", "-0.0", "1.0", "2"])
         .succeeds()
-        .stdout_is("-0.0\n01.0\n02.0\n")
-        .no_stderr();
+        .stdout_only("-0.0\n01.0\n02.0\n");
     new_ucmd!()
         .args(&["-w", "-0.0", "1.0", "2.0"])
         .succeeds()
-        .stdout_is("-0.0\n01.0\n02.0\n")
-        .no_stderr();
+        .stdout_only("-0.0\n01.0\n02.0\n");
 }
 
 #[test]
@@ -393,98 +383,80 @@ fn test_width_negative_zero_scientific_notation() {
     new_ucmd!()
         .args(&["-w", "-0e0", "1"])
         .succeeds()
-        .stdout_is("-0\n01\n")
-        .no_stderr();
+        .stdout_only("-0\n01\n");
     new_ucmd!()
         .args(&["-w", "-0e0", "1", "2"])
         .succeeds()
-        .stdout_is("-0\n01\n02\n")
-        .no_stderr();
+        .stdout_only("-0\n01\n02\n");
     new_ucmd!()
         .args(&["-w", "-0e0", "1", "2.0"])
         .succeeds()
-        .stdout_is("-0\n01\n02\n")
-        .no_stderr();
+        .stdout_only("-0\n01\n02\n");
 
     new_ucmd!()
         .args(&["-w", "-0e+1", "1"])
         .succeeds()
-        .stdout_is("-00\n001\n")
-        .no_stderr();
+        .stdout_only("-00\n001\n");
     new_ucmd!()
         .args(&["-w", "-0e+1", "1", "2"])
         .succeeds()
-        .stdout_is("-00\n001\n002\n")
-        .no_stderr();
+        .stdout_only("-00\n001\n002\n");
     new_ucmd!()
         .args(&["-w", "-0e+1", "1", "2.0"])
         .succeeds()
-        .stdout_is("-00\n001\n002\n")
-        .no_stderr();
+        .stdout_only("-00\n001\n002\n");
 
     new_ucmd!()
         .args(&["-w", "-0.000e0", "1"])
         .succeeds()
-        .stdout_is("-0.000\n01.000\n")
-        .no_stderr();
+        .stdout_only("-0.000\n01.000\n");
     new_ucmd!()
         .args(&["-w", "-0.000e0", "1", "2"])
         .succeeds()
-        .stdout_is("-0.000\n01.000\n02.000\n")
-        .no_stderr();
+        .stdout_only("-0.000\n01.000\n02.000\n");
     new_ucmd!()
         .args(&["-w", "-0.000e0", "1", "2.0"])
         .succeeds()
-        .stdout_is("-0.000\n01.000\n02.000\n")
-        .no_stderr();
+        .stdout_only("-0.000\n01.000\n02.000\n");
 
     new_ucmd!()
         .args(&["-w", "-0.000e-2", "1"])
         .succeeds()
-        .stdout_is("-0.00000\n01.00000\n")
-        .no_stderr();
+        .stdout_only("-0.00000\n01.00000\n");
     new_ucmd!()
         .args(&["-w", "-0.000e-2", "1", "2"])
         .succeeds()
-        .stdout_is("-0.00000\n01.00000\n02.00000\n")
-        .no_stderr();
+        .stdout_only("-0.00000\n01.00000\n02.00000\n");
     new_ucmd!()
         .args(&["-w", "-0.000e-2", "1", "2.0"])
         .succeeds()
-        .stdout_is("-0.00000\n01.00000\n02.00000\n")
-        .no_stderr();
+        .stdout_only("-0.00000\n01.00000\n02.00000\n");
 
     new_ucmd!()
         .args(&["-w", "-0.000e5", "1"])
         .succeeds()
-        .stdout_is("-000000\n0000001\n")
-        .no_stderr();
+        .stdout_only("-000000\n0000001\n");
     new_ucmd!()
         .args(&["-w", "-0.000e5", "1", "2"])
         .succeeds()
-        .stdout_is("-000000\n0000001\n0000002\n")
-        .no_stderr();
+        .stdout_only("-000000\n0000001\n0000002\n");
     new_ucmd!()
         .args(&["-w", "-0.000e5", "1", "2.0"])
         .succeeds()
-        .stdout_is("-000000\n0000001\n0000002\n")
-        .no_stderr();
+        .stdout_only("-000000\n0000001\n0000002\n");
 
     new_ucmd!()
         .args(&["-w", "-0.000e5", "1"])
         .succeeds()
-        .stdout_is("-000000\n0000001\n")
-        .no_stderr();
+        .stdout_only("-000000\n0000001\n");
     new_ucmd!()
         .args(&["-w", "-0.000e5", "1", "2"])
         .succeeds()
-        .stdout_is("-000000\n0000001\n0000002\n")
-        .no_stderr();
+        .stdout_only("-000000\n0000001\n0000002\n");
     new_ucmd!()
         .args(&["-w", "-0.000e5", "1", "2.0"])
         .succeeds()
-        .stdout_is("-000000\n0000001\n0000002\n")
-        .no_stderr();
+        .stdout_only("-000000\n0000001\n0000002\n");
 }
 
 #[test]
@@ -492,14 +464,12 @@ fn test_width_decimal_scientific_notation_increment() {
     new_ucmd!()
         .args(&["-w", ".1", "1e-2", ".11"])
         .succeeds()
-        .stdout_is("0.10\n0.11\n")
-        .no_stderr();
+        .stdout_only("0.10\n0.11\n");
 
     new_ucmd!()
         .args(&["-w", ".0", "1.500e-1", ".2"])
         .succeeds()
-        .stdout_is("0.0000\n0.1500\n")
-        .no_stderr();
+        .stdout_only("0.0000\n0.1500\n");
 }
 
 /// Test that trailing zeros in the start argument contribute to precision.
@@ -508,8 +478,7 @@ fn test_width_decimal_scientific_notation_trailing_zeros_start() {
     new_ucmd!()
         .args(&["-w", ".1000", "1e-2", ".11"])
         .succeeds()
-        .stdout_is("0.1000\n0.1100\n")
-        .no_stderr();
+        .stdout_only("0.1000\n0.1100\n");
 }
 
 /// Test that trailing zeros in the increment argument contribute to precision.
@@ -518,8 +487,7 @@ fn test_width_decimal_scientific_notation_trailing_zeros_increment() {
     new_ucmd!()
         .args(&["-w", "1e-1", "0.0100", ".11"])
         .succeeds()
-        .stdout_is("0.1000\n0.1100\n")
-        .no_stderr();
+        .stdout_only("0.1000\n0.1100\n");
 }
 
 #[test]
@@ -527,8 +495,7 @@ fn test_width_negative_decimal_notation() {
     new_ucmd!()
         .args(&["-w", "-.1", ".1", ".11"])
         .succeeds()
-        .stdout_is("-0.1\n00.0\n00.1\n")
-        .no_stderr();
+        .stdout_only("-0.1\n00.0\n00.1\n");
 }
 
 #[test]
@@ -536,22 +503,19 @@ fn test_width_negative_scientific_notation() {
     new_ucmd!()
         .args(&["-w", "-1e-3", "1"])
         .succeeds()
-        .stdout_is("-0.001\n00.999\n")
-        .no_stderr();
+        .stdout_only("-0.001\n00.999\n");
     new_ucmd!()
         .args(&["-w", "-1.e-3", "1"])
         .succeeds()
-        .stdout_is("-0.001\n00.999\n")
-        .no_stderr();
+        .stdout_only("-0.001\n00.999\n");
     new_ucmd!()
         .args(&["-w", "-1.0e-4", "1"])
         .succeeds()
-        .stdout_is("-0.00010\n00.99990\n")
-        .no_stderr();
+        .stdout_only("-0.00010\n00.99990\n");
     new_ucmd!()
         .args(&["-w", "-.1e2", "10", "100"])
         .succeeds()
-        .stdout_is(
+        .stdout_only(
             "-010
 0000
 0010
@@ -565,12 +529,11 @@ fn test_width_negative_scientific_notation() {
 0090
 0100
 ",
-        )
-        .no_stderr();
+        );
     new_ucmd!()
         .args(&["-w", "-0.1e2", "10", "100"])
         .succeeds()
-        .stdout_is(
+        .stdout_only(
             "-010
 0000
 0010
@@ -584,8 +547,7 @@ fn test_width_negative_scientific_notation() {
 0090
 0100
 ",
-        )
-        .no_stderr();
+        );
 }
 
 /// Test that trailing zeros in the end argument do not contribute to width.
@@ -594,8 +556,7 @@ fn test_width_decimal_scientific_notation_trailing_zeros_end() {
     new_ucmd!()
         .args(&["-w", "1e-1", "1e-2", ".1100"])
         .succeeds()
-        .stdout_is("0.10\n0.11\n")
-        .no_stderr();
+        .stdout_only("0.10\n0.11\n");
 }
 
 #[test]
@@ -603,8 +564,7 @@ fn test_width_floats() {
     new_ucmd!()
         .args(&["-w", "9.0", "10.0"])
         .succeeds()
-        .stdout_is("09.0\n10.0\n")
-        .no_stderr();
+        .stdout_only("09.0\n10.0\n");
 }
 
 // TODO This is duplicated from `test_yes.rs`; refactor them.
@@ -623,11 +583,21 @@ fn test_neg_inf() {
     run(&["--", "-inf", "0"], b"-inf\n-inf\n-inf\n");
 }
 
+#[test]
+fn test_neg_infinity() {
+    run(&["--", "-infinity", "0"], b"-inf\n-inf\n-inf\n");
+}
+
 #[test]
 fn test_inf() {
     run(&["inf"], b"1\n2\n3\n");
 }
 
+#[test]
+fn test_infinity() {
+    run(&["infinity"], b"1\n2\n3\n");
+}
+
 #[test]
 fn test_inf_width() {
     run(
@@ -646,11 +616,7 @@ fn test_neg_inf_width() {
 
 #[test]
 fn test_ignore_leading_whitespace() {
-    new_ucmd!()
-        .arg("   1")
-        .succeeds()
-        .stdout_is("1\n")
-        .no_stderr();
+    new_ucmd!().arg("   1").succeeds().stdout_only("1\n");
 }
 
 #[test]
@@ -669,8 +635,7 @@ fn test_negative_zero_int_start_float_increment() {
     new_ucmd!()
         .args(&["-0", "0.1", "0.1"])
         .succeeds()
-        .stdout_is("-0.0\n0.1\n")
-        .no_stderr();
+        .stdout_only("-0.0\n0.1\n");
 }
 
 #[test]
@@ -678,7 +643,7 @@ fn test_float_precision_increment() {
     new_ucmd!()
         .args(&["999", "0.1", "1000.1"])
         .succeeds()
-        .stdout_is(
+        .stdout_only(
             "999.0
 999.1
 999.2
@@ -692,8 +657,7 @@ fn test_float_precision_increment() {
 1000.0
 1000.1
 ",
-        )
-        .no_stderr();
+        );
 }
 
 /// Test for floating point precision issues.
@@ -702,8 +666,7 @@ fn test_negative_increment_decimal() {
     new_ucmd!()
         .args(&["0.1", "-0.1", "-0.2"])
         .succeeds()
-        .stdout_is("0.1\n0.0\n-0.1\n-0.2\n")
-        .no_stderr();
+        .stdout_only("0.1\n0.0\n-0.1\n-0.2\n");
 }
 
 #[test]
@@ -711,8 +674,7 @@ fn test_zero_not_first() {
     new_ucmd!()
         .args(&["-w", "-0.1", "0.1", "0.1"])
         .succeeds()
-        .stdout_is("-0.1\n00.0\n00.1\n")
-        .no_stderr();
+        .stdout_only("-0.1\n00.0\n00.1\n");
 }
 
 #[test]
@@ -720,8 +682,7 @@ fn test_rounding_end() {
     new_ucmd!()
         .args(&["1", "-1", "0.1"])
         .succeeds()
-        .stdout_is("1\n")
-        .no_stderr();
+        .stdout_only("1\n");
 }
 
 #[test]
@@ -737,7 +698,7 @@ fn test_parse_error_hex() {
     new_ucmd!()
         .arg("0xlmnop")
         .fails()
-        .usage_error("invalid hexadecimal argument: '0xlmnop'");
+        .usage_error("invalid floating point argument: '0xlmnop'");
 }
 
 #[test]
@@ -748,6 +709,43 @@ fn test_format_option() {
         .stdout_only("0.00\n0.10\n0.20\n0.30\n0.40\n0.50\n");
 }
 
+#[test]
+#[ignore = "Need issue #2660 to be fixed"]
+fn test_auto_precision() {
+    new_ucmd!()
+        .args(&["1", "0x1p-1", "2"])
+        .succeeds()
+        .stdout_only("1\n1.5\n2\n");
+}
+
+#[test]
+#[ignore = "Need issue #3318 to be fixed"]
+fn test_undefined() {
+    new_ucmd!()
+        .args(&["1e-9223372036854775808"])
+        .succeeds()
+        .no_output();
+}
+
+#[test]
+fn test_invalid_float_point_fail_properly() {
+    new_ucmd!()
+        .args(&["66000e000000000000000000000000000000000000000000000000000009223372036854775807"])
+        .fails()
+        .no_stdout()
+        .usage_error("invalid floating point argument: '66000e000000000000000000000000000000000000000000000000000009223372036854775807'");
+    new_ucmd!()
+        .args(&["-1.1e9223372036854775807"])
+        .fails()
+        .no_stdout()
+        .usage_error("invalid floating point argument: '-1.1e9223372036854775807'");
+    new_ucmd!()
+        .args(&["-.1e9223372036854775807"])
+        .fails()
+        .no_stdout()
+        .usage_error("invalid floating point argument: '-.1e9223372036854775807'");
+}
+
 #[test]
 fn test_invalid_zero_increment_value() {
     new_ucmd!()
@@ -756,3 +754,164 @@ fn test_invalid_zero_increment_value() {
         .no_stdout()
         .usage_error("invalid Zero increment value: '0'");
 }
+
+#[test]
+fn test_power_of_ten_display() {
+    new_ucmd!()
+        .args(&["-f", "%.2g", "10", "10"])
+        .succeeds()
+        .stdout_only("10\n");
+}
+
+#[test]
+fn test_default_g_precision() {
+    new_ucmd!()
+        .args(&["-f", "%010g", "1e5", "1e5"])
+        .succeeds()
+        .stdout_only("0000100000\n");
+    new_ucmd!()
+        .args(&["-f", "%010g", "1e6", "1e6"])
+        .succeeds()
+        .stdout_only("000001e+06\n");
+}
+
+#[test]
+fn test_invalid_format() {
+    new_ucmd!()
+        .args(&["-f", "%%g", "1"])
+        .fails()
+        .no_stdout()
+        .stderr_contains("format '%%g' has no % directive");
+    new_ucmd!()
+        .args(&["-f", "%g%g", "1"])
+        .fails()
+        .no_stdout()
+        .stderr_contains("format '%g%g' has too many % directives");
+    new_ucmd!()
+        .args(&["-f", "%g%", "1"])
+        .fails()
+        .no_stdout()
+        .stderr_contains("format '%g%' has too many % directives");
+    new_ucmd!()
+        .args(&["-f", "%", "1"])
+        .fails()
+        .no_stdout()
+        .stderr_contains("format '%' ends in %");
+}
+
+#[test]
+fn test_parse_scientific_zero() {
+    new_ucmd!()
+        .args(&["0e15", "1"])
+        .succeeds()
+        .stdout_only("0\n1\n");
+    new_ucmd!()
+        .args(&["0.0e15", "1"])
+        .succeeds()
+        .stdout_only("0\n1\n");
+    new_ucmd!()
+        .args(&["0", "1"])
+        .succeeds()
+        .stdout_only("0\n1\n");
+    new_ucmd!()
+        .args(&["-w", "0e15", "1"])
+        .succeeds()
+        .stdout_only("0000000000000000\n0000000000000001\n");
+    new_ucmd!()
+        .args(&["-w", "0.0e15", "1"])
+        .succeeds()
+        .stdout_only("0000000000000000\n0000000000000001\n");
+    new_ucmd!()
+        .args(&["-w", "0", "1"])
+        .succeeds()
+        .stdout_only("0\n1\n");
+}
+
+#[test]
+fn test_parse_valid_hexadecimal_float_two_args() {
+    let test_cases = [
+        (["0x1p-1", "2"], "0.5\n1.5\n"),
+        (["0x.8p16", "32768"], "32768\n"),
+        (["0xffff.4p-4", "4096"], "4095.95\n"),
+        (["0xA.A9p-1", "6"], "5.33008\n"),
+        (["0xa.a9p-1", "6"], "5.33008\n"),
+        (["0xffffffffffp-30", "1024"], "1024\n"), // spell-checker:disable-line
+    ];
+
+    for (input_arguments, expected_output) in &test_cases {
+        new_ucmd!()
+            .args(input_arguments)
+            .succeeds()
+            .stdout_only(expected_output);
+    }
+}
+
+#[test]
+fn test_parse_valid_hexadecimal_float_three_args() {
+    let test_cases = [
+        (["0x3.4p-1", "0x4p-1", "4"], "1.625\n3.625\n"),
+        (
+            ["-0x.ep-3", "-0x.1p-3", "-0x.fp-3"],
+            "-0.109375\n-0.117188\n",
+        ),
+    ];
+
+    for (input_arguments, expected_output) in &test_cases {
+        new_ucmd!()
+            .args(input_arguments)
+            .succeeds()
+            .stdout_only(expected_output);
+    }
+}
+
+#[test]
+fn test_parse_float_gnu_coreutils() {
+    // some values from GNU coreutils tests
+    new_ucmd!()
+        .args(&[".89999", "1e-7", ".8999901"])
+        .succeeds()
+        .stdout_only("0.8999900\n0.8999901\n");
+
+    new_ucmd!()
+        .args(&["0", "0.000001", "0.000003"])
+        .succeeds()
+        .stdout_only("0.000000\n0.000001\n0.000002\n0.000003\n");
+}
+
+#[test]
+fn test_parse_out_of_bounds_exponents() {
+    // The value 1e-9223372036854775808 is used in GNU Coreutils and BigDecimal tests to verify
+    // overflows and undefined behavior. Let's test the value too.
+    new_ucmd!()
+        .args(&["1e-9223372036854775808"])
+        .succeeds()
+        .stdout_only("");
+}
+
+#[ignore]
+#[test]
+fn test_parse_valid_hexadecimal_float_format_issues() {
+    // These tests detect differences in the representation of floating-point values with GNU seq.
+    // There are two key areas to investigate:
+    //
+    // 1. GNU seq uses long double (80-bit) types for values, while the current implementation
+    // relies on f64 (64-bit). This can lead to differences due to varying precision. However, it's
+    // likely not the primary cause, as even double (64-bit) values can differ when compared to
+    // f64.
+    //
+    // 2. GNU seq uses the %Lg format specifier for printing (see the "get_default_format" function
+    // ). It appears that Rust lacks a direct equivalent for this format. Additionally, %Lg
+    // can use %f (floating) or %e (scientific) depending on the precision. There also seem to be
+    // some differences in the behavior of C and Rust when displaying floating-point or scientific
+    // notation, at least without additional configuration.
+    //
+    // It makes sense to begin by experimenting with formats and attempting to replicate
+    // the printf("%Lg",...) behavior. Another area worth investigating is glibc, as reviewing its
+    // code may help uncover additional corner cases or test data that could reveal more issues.
+
+    //Test output: 0.00000000992804416455328464508056640625
+    new_ucmd!()
+        .args(&["0xa.a9p-30", "1"])
+        .succeeds()
+        .stdout_only("9.92804e-09\n1\n");
+}
