@@ -2,10 +2,10 @@
 //
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
+
 use crate::common::util::{vec_of_size, TestScenario};
 
 // spell-checker:ignore (flags) lwmcL clmwL ; (path) bogusfile emptyfile manyemptylines moby notrailingnewline onelongemptyline onelongword weirdchars
-
 #[test]
 fn test_invalid_arg() {
     new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
@@ -71,7 +71,7 @@ fn test_utf8_words() {
         .arg("-w")
         .pipe_in_fixture("UTF_8_weirdchars.txt")
         .run()
-        .stdout_is("87\n");
+        .stdout_is("89\n");
 }
 
 #[test]
@@ -80,7 +80,7 @@ fn test_utf8_line_length_words() {
         .arg("-Lw")
         .pipe_in_fixture("UTF_8_weirdchars.txt")
         .run()
-        .stdout_is("     87      48\n");
+        .stdout_is("     89      48\n");
 }
 
 #[test]
@@ -98,7 +98,7 @@ fn test_utf8_line_length_chars_words() {
         .arg("-Lmw")
         .pipe_in_fixture("UTF_8_weirdchars.txt")
         .run()
-        .stdout_is("     87     442      48\n");
+        .stdout_is("     89     442      48\n");
 }
 
 #[test]
@@ -143,7 +143,7 @@ fn test_utf8_chars_words() {
         .arg("-mw")
         .pipe_in_fixture("UTF_8_weirdchars.txt")
         .run()
-        .stdout_is("     87     442\n");
+        .stdout_is("     89     442\n");
 }
 
 #[test]
@@ -161,7 +161,7 @@ fn test_utf8_line_length_lines_words() {
         .arg("-Llw")
         .pipe_in_fixture("UTF_8_weirdchars.txt")
         .run()
-        .stdout_is("     25      87      48\n");
+        .stdout_is("     25      89      48\n");
 }
 
 #[test]
@@ -179,7 +179,7 @@ fn test_utf8_lines_words_chars() {
         .arg("-mlw")
         .pipe_in_fixture("UTF_8_weirdchars.txt")
         .run()
-        .stdout_is("     25      87     442\n");
+        .stdout_is("     25      89     442\n");
 }
 
 #[test]
@@ -197,7 +197,17 @@ fn test_utf8_all() {
         .arg("-lwmcL")
         .pipe_in_fixture("UTF_8_weirdchars.txt")
         .run()
-        .stdout_is("     25      87     442     513      48\n");
+        .stdout_is("     25      89     442     513      48\n");
+}
+
+#[test]
+fn test_ascii_control() {
+    // GNU coreutils "d1" test
+    new_ucmd!()
+        .arg("-w")
+        .pipe_in(*b"\x01\n")
+        .run()
+        .stdout_is("1\n");
 }
 
 #[test]
@@ -243,6 +253,14 @@ fn test_single_only_lines() {
         .stdout_is("18 moby_dick.txt\n");
 }
 
+#[test]
+fn test_single_only_bytes() {
+    new_ucmd!()
+        .args(&["-c", "lorem_ipsum.txt"])
+        .run()
+        .stdout_is("772 lorem_ipsum.txt\n");
+}
+
 #[test]
 fn test_single_all_counts() {
     new_ucmd!()
@@ -265,6 +283,32 @@ fn test_gnu_compatible_quotation() {
         .stdout_is("0 0 0 'some-dir1/12'$'\\n''34.txt'\n");
 }
 
+#[cfg(feature = "test_risky_names")]
+#[test]
+fn test_non_unicode_names() {
+    let scene = TestScenario::new(util_name!());
+    let target1 = uucore::os_str_from_bytes(b"some-dir1/1\xC0\n.txt")
+        .expect("Only unix platforms can test non-unicode names");
+    let target2 = uucore::os_str_from_bytes(b"some-dir1/2\xC0\t.txt")
+        .expect("Only unix platforms can test non-unicode names");
+    let at = &scene.fixtures;
+    at.mkdir("some-dir1");
+    at.touch(&target1);
+    at.touch(&target2);
+    scene
+        .ucmd()
+        .args(&[target1, target2])
+        .run()
+        .stdout_is_bytes(
+            [
+                b"0 0 0 'some-dir1/1'$'\\300\\n''.txt'\n".to_vec(),
+                b"0 0 0 some-dir1/2\xC0\t.txt\n".to_vec(),
+                b"0 0 0 total\n".to_vec(),
+            ]
+            .concat(),
+        );
+}
+
 #[test]
 fn test_multiple_default() {
     new_ucmd!()
@@ -339,6 +383,19 @@ fn test_file_one_long_word() {
 /// bytes are displayed.
 #[test]
 fn test_file_bytes_dictate_width() {
+    // . is a directory, so minimum_width should get set to 7
+    #[cfg(not(windows))]
+    const STDOUT: &str = concat!(
+        "      0       0       0 emptyfile.txt\n",
+        "      0       0       0 .\n",
+        "      0       0       0 total\n",
+    );
+    #[cfg(windows)]
+    const STDOUT: &str = concat!(
+        "      0       0       0 emptyfile.txt\n",
+        "      0       0       0 total\n",
+    );
+
     // This file has 10,001 bytes. Five digits are required to
     // represent that. Even though the number of lines is 1 and the
     // number of words is 0, each of those counts is formatted with
@@ -366,18 +423,6 @@ fn test_file_bytes_dictate_width() {
             "  18  166 1074 total\n",
         ));
 
-    // . is a directory, so minimum_width should get set to 7
-    #[cfg(not(windows))]
-    const STDOUT: &str = concat!(
-        "      0       0       0 emptyfile.txt\n",
-        "      0       0       0 .\n",
-        "      0       0       0 total\n",
-    );
-    #[cfg(windows)]
-    const STDOUT: &str = concat!(
-        "      0       0       0 emptyfile.txt\n",
-        "      0       0       0 total\n",
-    );
     new_ucmd!()
         .args(&["-lwc", "emptyfile.txt", "."])
         .run()
@@ -419,11 +464,24 @@ fn test_files_from_pseudo_filesystem() {
     use pretty_assertions::assert_ne;
     let result = new_ucmd!().arg("-c").arg("/proc/cpuinfo").succeeds();
     assert_ne!(result.stdout_str(), "0 /proc/cpuinfo\n");
+
+    // the following block fails on Android with a "Permission denied" error
+    #[cfg(target_os = "linux")]
+    {
+        let (at, mut ucmd) = at_and_ucmd!();
+        let result = ucmd.arg("-c").arg("/sys/kernel/profiling").succeeds();
+        let actual = at.read("/sys/kernel/profiling").len();
+        assert_eq!(
+            result.stdout_str(),
+            format!("{actual} /sys/kernel/profiling\n")
+        );
+    }
 }
 
 #[test]
 fn test_files0_disabled_files_argument() {
-    const MSG: &str = "file operands cannot be combined with --files0-from";
+    const MSG: &str =
+        "extra operand 'lorem_ipsum.txt'\nfile operands cannot be combined with --files0-from";
     new_ucmd!()
         .args(&["--files0-from=files0_list.txt"])
         .arg("lorem_ipsum.txt")
@@ -500,6 +558,10 @@ fn test_total_auto() {
         .args(&["lorem_ipsum.txt", "--total=auto"])
         .run()
         .stdout_is(" 13 109 772 lorem_ipsum.txt\n");
+    new_ucmd!()
+        .args(&["lorem_ipsum.txt", "--tot=au"])
+        .run()
+        .stdout_is(" 13 109 772 lorem_ipsum.txt\n");
 
     new_ucmd!()
         .args(&["lorem_ipsum.txt", "moby_dick.txt", "--total=auto"])
@@ -520,6 +582,13 @@ fn test_total_always() {
             " 13 109 772 lorem_ipsum.txt\n",
             " 13 109 772 total\n",
         ));
+    new_ucmd!()
+        .args(&["lorem_ipsum.txt", "--total=al"])
+        .run()
+        .stdout_is(concat!(
+            " 13 109 772 lorem_ipsum.txt\n",
+            " 13 109 772 total\n",
+        ));
 
     new_ucmd!()
         .args(&["lorem_ipsum.txt", "moby_dick.txt", "--total=always"])
@@ -545,6 +614,13 @@ fn test_total_never() {
             "  13  109  772 lorem_ipsum.txt\n",
             "  18  204 1115 moby_dick.txt\n",
         ));
+    new_ucmd!()
+        .args(&["lorem_ipsum.txt", "moby_dick.txt", "--total=n"])
+        .run()
+        .stdout_is(concat!(
+            "  13  109  772 lorem_ipsum.txt\n",
+            "  18  204 1115 moby_dick.txt\n",
+        ));
 }
 
 #[test]
@@ -558,6 +634,10 @@ fn test_total_only() {
         .args(&["lorem_ipsum.txt", "moby_dick.txt", "--total=only"])
         .run()
         .stdout_is("31 313 1887\n");
+    new_ucmd!()
+        .args(&["lorem_ipsum.txt", "moby_dick.txt", "--t=o"])
+        .run()
+        .stdout_is("31 313 1887\n");
 }
 
 #[test]
@@ -684,6 +764,10 @@ fn files0_from_dir() {
             concat!("wc: cannot open ", $p, " for reading: Permission denied\n")
         };
     }
+    #[cfg(windows)]
+    const DOT_ERR: &str = dir_err!("'.'");
+    #[cfg(not(windows))]
+    const DOT_ERR: &str = dir_err!(".");
 
     new_ucmd!()
         .args(&["--files0-from=dir with spaces"])
@@ -691,10 +775,6 @@ fn files0_from_dir() {
         .stderr_only(dir_err!("'dir with spaces'"));
 
     // Those contexts have different rules about quoting in errors...
-    #[cfg(windows)]
-    const DOT_ERR: &str = dir_err!("'.'");
-    #[cfg(not(windows))]
-    const DOT_ERR: &str = dir_err!(".");
     new_ucmd!()
         .args(&["--files0-from=."])
         .fails()
@@ -708,3 +788,16 @@ fn files0_from_dir() {
         .fails()
         .stderr_only(dir_err!("-"));
 }
+
+#[test]
+fn test_args_override() {
+    new_ucmd!()
+        .args(&["-ll", "-l", "alice_in_wonderland.txt"])
+        .run()
+        .stdout_is("5 alice_in_wonderland.txt\n");
+
+    new_ucmd!()
+        .args(&["--total=always", "--total=never", "alice_in_wonderland.txt"])
+        .run()
+        .stdout_is("  5  57 302 alice_in_wonderland.txt\n");
+}
